"""Wave pattern utilities (parallel lines, sawtooth, sine, square, etc.).

Example:
    from waves.parallel_lines import create_parallel_lines
    img = create_parallel_lines(width=600, height=400, angle=30)
    img.show()
"""
