"""
Grid drawing utilities (hexagon grids, shapes grids, etc.).

Example:
    from grid.hexagon_grid import create_hexagon_grid
    img = create_hexagon_grid(width=600, height=600, hexagon_size=30)
    img.show()
"""
